//This Sketch is for an Arduino Uno Rev 3 and a Cytron L298P Motor Shield.
//This Sketch is based on the basic YOGI Sketch found at http://www.duino-robotics.com/obstacle-avoidance.html.
//Modified to drive motors instead of servos.

#include <NewPing.h>   //Include NewPing Library

#define mtr1dirPin 4      //Motor 1 Direction connected to pin 4
#define mtr1enPin 5       //Motor 1 Enable connected to pin 5
#define mtr2enPin 6       //Motor 2 Direction connected to pin 6
#define mtr2dirPin 7      //Motor 2 Enable connected to pin 7
#define ECHO_PIN     12   //Echo pin of Ultrasonic sensor connected to pin 12
#define TRIGGER_PIN  13   //Trigger pin of Ultrasonic sensor connected to pin 13
#define MAX_DISTANCE 100  //The maximum distance we want the sensor to look for is 1m


NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE);  //Create Ultrasonic sensor object

unsigned int time;            //Variable to store how long it takes for the ultrasonic wave to come back
int distance;                 //Variable to store the distance calculated from the sensor
int triggerDistance = 25;     //The distance we want the robot to look for a new path
int fDistance;                //Variable to store the distance in front of the robot
int lDistance;                //Variable to store the distance on the left side of the robot
int rDistance;                //Variable to store the distance on the right side of the robot

void setup() 
{ 
  pinMode(mtr1dirPin, OUTPUT);    // Set Motor 1 Direction Pin 4 to Output Mode
  pinMode(mtr1enPin, OUTPUT);     // Set Motor 1 Enable Pin 5 to Output Mode
  pinMode(mtr2dirPin, OUTPUT);    // Set Motor 2 Direction Pin 6 to Output Mode
  pinMode(mtr2enPin, OUTPUT);     // Set Motor 2 Enable Pin 7 to Output Mode
} 

void loop()
{
  scan();                                //Get the distance retrieved
  fDistance = distance;                  //Set that distance to the front distance
  if(fDistance < triggerDistance){       //If there is something closer than 25cm in front of us
    moveBackward();                      //Move Backward for 1 second
    delay(1000); 
    moveStop();
    delay(500);
    moveRight();                         //Turn Right for half 2 seconds
    delay(2000);
    moveStop();                          //Stop
    delay(500);
    scan();                              //Take a reading
    rDistance = distance;                //Store that to the distance on the right side
    moveLeft();
    delay(4000);                         //Turn left for 4 second
    moveStop();                          //Stop
    delay(500);
    scan();                              //Take a reading
    lDistance = distance;                //Store that to the distance on the left side
    if(lDistance < rDistance){           //If the distance on the left is smaller than that of the right
      moveRight();                       //Move right for 2 second
      delay(4000);
      moveStop();
      delay(500);
      moveForward();                     //Then move forward
    }
    else{
      moveForward();                     //If the left side is larger than the right side move forward
    }
  }
  else{
    moveForward();                       //If there is nothing infront of the robot move forward
  }
}

void scan(){
  time = sonar.ping();                  //Send out a ping and store the time it took for it to come back in the time variable
  distance = time / US_ROUNDTRIP_CM;    //Convert that time into a distance
  if(distance == 0){                    //If no ping was recieved
    distance = 100;                     //Set the distance to max
  }
  delay(10);
}

void moveForward(){

  digitalWrite(mtr1enPin, HIGH);
  digitalWrite(mtr1dirPin, LOW);
  digitalWrite(mtr2enPin, HIGH);
  digitalWrite(mtr2dirPin, LOW); 
}

void moveBackward(){

  digitalWrite(mtr1enPin, HIGH);
  digitalWrite(mtr1dirPin, HIGH);
  digitalWrite(mtr2enPin, HIGH);
  digitalWrite(mtr2dirPin, HIGH);
}

void moveRight(){

  digitalWrite(mtr1enPin, HIGH);
  digitalWrite(mtr1dirPin, HIGH);
  digitalWrite(mtr2enPin, HIGH);
  digitalWrite(mtr2dirPin, LOW);
}

void moveLeft(){

  digitalWrite(mtr1enPin, HIGH);
  digitalWrite(mtr1dirPin, LOW);
  digitalWrite(mtr2enPin, HIGH);
  digitalWrite(mtr2dirPin, HIGH);
}

void moveStop(){

  digitalWrite(mtr1enPin, LOW);
  digitalWrite(mtr1dirPin, LOW);
  digitalWrite(mtr2enPin, LOW);
  digitalWrite(mtr2dirPin, LOW);
}


